(

~bergerBusArray = 6.collect({Bus.audio(s, 1)});

Event.addEventType(\osc, {
	if (~addr.postln.notNil) {
		~msg.postln;
		~addr.sendMsg(~path, *~msg);
	};
});

//create synth defs
~bergerCreateSynthsForLive = {arg b;
	// this creates a different waveform for each sonification of each part
	Buffer.freeAll(s);
	b = 8.collect({var buf = Buffer.alloc(s, 512, 1); buf.sine1(1.0 / 5.collect({arg i; pow(i + 1, 5.0.rand + 1)}), true, true, true)});

	SynthDef(\berger, {arg freq, amp = 0.2, del = 5, gate = 1, sustain = 1, buf = 0, out = 0;
		Out.ar(out, Osc.ar(Select.kr(buf, b), freq, 0, amp) * EnvGen.kr(Env.adsr(0.05, sustain, 0.25, 0.1), gate, 1, 0, 1, doneAction: 2));
	}).add;
};

//sonify musical data
~bergerCreatePatternsForLive = {arg seqs, group, addr;
	var pBinds, transportPattern;
	pBinds = seqs.collect({arg r, i;
		Pbind(\instrument, \berger,
			//\buf, Pseq((i - ((0..(r.slice(nil, 2).flat.size - 1)) / 10).trunc.asInteger) % 8),
			\group, group,
			\buf, i,
			\out, ~bergerBusArray[i],
			//\freq, Pseq(55 * r[1]),
			\freq, Pseq((55 * r[1]).cpsmidi.round(0.5).midicps),
			\dur, Pseq(r[0]),
			\sustain, Pseq(r[0]),
			\amp, Pseq(r[2] / 1) //4)
		)
	});

	transportPattern = Pbind(
		\type, \osc,
		\addr, addr,
		\path, "/playing",
		\msg, Pseq(seqs[0][6].collect({arg beats, measure; beats.asInteger.collect({arg beat; [measure + 1, beat + 1]})}).flatten, 1),
		\dur, 1
	);

	Ppar(pBinds.add(transportPattern));
};


SynthDef(\mixer, {arg freq, gate = 1, sustain, amp, dur, out;
	var nameSpaces, sigs;

	sigs = [~bergerBusArray].collect({arg busArray, i;
		var nameSpace, sig;
		nameSpace = ['berger', 'sine', 'bass', 'hdust', 'sampler'][i];
		sig = busArray.collect({arg bus, c; In.ar(bus, 1) * NamedControl.kr(\ ++ nameSpace ++ '_volume_' ++ c, 1, 0.1)});
		sig = sig.collect({arg channel, c; Pan2.ar(channel, NamedControl.kr(\ ++ nameSpace ++ '_pan_' ++ c, i / (busArray.size - 1), 0.1) * 2 - 1)});
		sig = sig.collect({arg channel, c; channel * NamedControl.kr(\ ++ nameSpace ++ '_mute_' ++ c, 1, 0.1)});
		sig = Mix.ar(sig) * pow(NamedControl.kr(\ ++ nameSpace ++ '_volume_master', 1, 0.5), 2);
	});

	sigs = Mix.ar(sigs / 6);
	Out.ar(0, sigs)
}).add;

)

(

var group, berger, mixer, measureDurs, player;

~bergerCreateSynthsForLive.value;

group = Group.new;
berger = Synth.tail(group, \berger);
mixer = Synth.tail(group, \mixer);

measureDurs = ~bergerMusic[0][6];

OSCdef(\mixer, {arg msg, time, addr, port;
	msg.postln;
	mixer.set((msg[1] ++ '_' ++ msg[2] ++ '_' ++ msg[3]), msg[4])
}, \mixer);

OSCdef(\transport, {arg msg, time, addr, port;
	msg.postln;
	if(msg[1].postln == 0, {
		group.set(\release, 2);
		group.set(\gate, 0);
		player.stop;
	}, {
		var patterns, stream, offset, offsetStream, terminationStream;
		patterns = ~bergerCreatePatternsForLive.value(~bergerMusic, group, addr);

		stream = patterns.asStream;
		offset = stream.fastForward(measureDurs.keep(msg[2].asInteger.postln - 1).sum.postln).postln;

		offsetStream = Routine({offset.wait});
		terminationStream = Routine({addr.sendMsg("/transport", 0)});

		player = EventStreamPlayer(offsetStream ++ stream ++ terminationStream);
		player.play;
	});
}, \transport);

)

/*
~bergerCreateSynthsForLive.value;
~patterns = ~bergerCreatePatternsForLive.value(~bergerMusic);
~patterns.play
~bergerMusic[0][6].collect({arg beats, measure; beats.asInteger.collect({arg beat; [measure + 1, beat + 1]})}).flatten;
*/
